/* eslint-disable react/display-name */
import { Accordion } from '@material-ui/core'
import Button from '@material-ui/core/Button'
import { FluidType } from 'enums'
import { mount } from 'enzyme'
import { PerformanceIndicator } from 'models'
import React from 'react'
import { Provider } from 'react-redux'
import { profileData } from 'tests/__mocks__/profileData.mock'
import {
  mockMonthlyForecastJanuaryTestProfile1,
  profileTypeData,
} from 'tests/__mocks__/profileType.mock'
import { createMockEcolyoStore, mockAnalysisState } from 'tests/__mocks__/store'
import { waitForComponentToPaint } from 'tests/__mocks__/testUtils'
import ProfileComparator from './ProfileComparator'

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

const mockGetMonthlyForecast = jest.fn()
jest.mock('services/profileType.service', () => {
  return jest.fn(() => ({
    getMonthlyForecast: mockGetMonthlyForecast,
  }))
})

const mockGetProfileTypeData = jest.fn()
jest.mock('services/profileTypeEntity.service', () => {
  return jest.fn(() => ({
    getProfileType: mockGetProfileTypeData,
  }))
})

jest.mock(
  'components/Analysis/ProfileComparator/ProfileComparatorRow',
  () => 'mock-profileComparatorRow'
)

const modifiedProfile = { ...profileData, isProfileTypeCompleted: true }
const store = createMockEcolyoStore({
  profile: modifiedProfile,
  analysis: mockAnalysisState,
})

const performanceIndicator = {
  compareValue: 160.42797399999998,
  percentageVariation: 0.026592126632478563,
  value: 156.161853,
}
const performanceIndicators = [
  {
    compareValue: 203.49,
    percentageVariation: 0.12261044768784712,
    value: 178.54,
  },
  {
    compareValue: 7926.82,
    percentageVariation: 0.020542916327102145,
    value: 7763.98,
  },
  {
    compareValue: 1316.46,
    percentageVariation: -0.0009191316105312541,
    value: 1317.67,
  },
]

describe('AnalysisConsumption component', () => {
  it('should be rendered correctly', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find(Accordion).exists()).toBeTruthy()
    expect(wrapper.find('mock-profileComparatorRow').length).toBe(4)
  })

  it('should be rendered correctly with no profil set', async () => {
    const mockAggregatedPerformanceIndicator: PerformanceIndicator = {
      compareValue: 160.42797399999998,
      percentageVariation: null,
      value: null,
    }
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={mockAggregatedPerformanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find(Button).exists()).toBeTruthy()
  })

  it('should be rendered correctly with null aggregated performance indicator', async () => {
    const mockAggregatedPerformanceIndicator: PerformanceIndicator = {
      compareValue: 160.42797399999998,
      percentageVariation: null,
      value: null,
    }
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={mockAggregatedPerformanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find(Accordion).exists()).toBeTruthy()
    expect(wrapper.find('mock-profileComparatorRow').length).toBe(4)
  })

  it('should be rendered correctly without fluid', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(
      wrapper.find('mock-profileComparatorRow').first().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should be rendered correctly with all fluids connected', async () => {
    const store = createMockEcolyoStore({
      profile: modifiedProfile,
      profileType: profileTypeData,
      global: {
        fluidTypes: [FluidType.ELECTRICITY, FluidType.WATER, FluidType.GAS],
      },
      analysis: {
        ...mockAnalysisState,
        analysisMonth: profileData.monthlyAnalysisDate,
      },
    })
    mockGetMonthlyForecast.mockReturnValue(
      mockMonthlyForecastJanuaryTestProfile1
    )
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(mockGetMonthlyForecast).toHaveBeenCalledWith(
      profileData.monthlyAnalysisDate.month - 1
    )
    expect(
      wrapper.find('mock-profileComparatorRow').first().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should be rendered correctly with 2 fluids connected', async () => {
    const store = createMockEcolyoStore({
      profile: modifiedProfile,
      global: { fluidTypes: [FluidType.ELECTRICITY, FluidType.WATER] },
      analysis: { analysisMonth: profileData.monthlyAnalysisDate },
    })
    mockGetMonthlyForecast.mockReturnValue(
      mockMonthlyForecastJanuaryTestProfile1
    )
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    expect(mockGetMonthlyForecast).toHaveBeenCalledWith(
      profileData.monthlyAnalysisDate.month - 1
    )
    expect(
      wrapper.find('mock-profileComparatorRow').first().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should redirect to profileType form when click on mui button', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <ProfileComparator
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
        />
      </Provider>
    )
    await waitForComponentToPaint(wrapper)
    wrapper.find(Button).first().simulate('click')
    expect(mockedNavigate).toHaveBeenCalledWith('/profileType')
  })
})
