import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
} from '@material-ui/core'
import Button from '@material-ui/core/Button'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import ProfileEditIcon from 'assets/icons/ico/profile-edit.svg'
import AnalysisIcon from 'assets/icons/visu/analysis/analysis.svg'
import PlaceHolderIcon from 'assets/icons/visu/analysis/no-profile-placeholder.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType } from 'enums'
import { MonthlyForecast, PerformanceIndicator, ProfileType } from 'models'
import React, { useCallback, useEffect, useState } from 'react'
import { useNavigate } from 'react-router-dom'
import ProfileTypeService from 'services/profileType.service'
import ProfileTypeEntityService from 'services/profileTypeEntity.service'
import { useAppSelector } from 'store/hooks'
import ProfileComparatorRow from './ProfileComparatorRow'
import './profileComparator.scss'

interface ProfileComparatorProps {
  aggregatedPerformanceIndicator: PerformanceIndicator
  performanceIndicators: PerformanceIndicator[]
}

const ProfileComparator = ({
  aggregatedPerformanceIndicator,
  performanceIndicators,
}: ProfileComparatorProps) => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const userPriceConsumption: number = aggregatedPerformanceIndicator.value || 0
  const {
    analysis: { analysisMonth },
    profile,
  } = useAppSelector(state => state.ecolyo)
  const [homePriceConsumption, setHomePriceConsumption] = useState<number>(0)
  const [forecast, setForecast] = useState<MonthlyForecast | null>(null)
  const [isLoading, setIsLoading] = useState<boolean>(true)

  const [activeAverageHome, setActiveAverageHome] = useState<boolean>(false)
  const toggleAccordion = () => {
    setActiveAverageHome(prev => !prev)
    if (!activeAverageHome) {
      setTimeout(() => {
        const content = document.querySelector('.consumption-electricity')
        if (content) {
          content.scrollIntoView({
            behavior: 'smooth',
            block: 'start',
          })
        }
      }, 300)
    }
  }

  const emptyFluidTypes: FluidType[] = []
  if (performanceIndicators.length === 0) {
    // If no indicators add all fluids for component placeholder
    emptyFluidTypes.push(FluidType.ELECTRICITY, FluidType.WATER, FluidType.GAS)
  } else {
    for (let i = 0; i < performanceIndicators.length; i++) {
      if (!performanceIndicators[i]?.value) {
        emptyFluidTypes.push(i)
      }
    }
  }

  const getTotalValueWithConnectedFluids = useCallback(
    (monthlyForecast: MonthlyForecast) => {
      if (performanceIndicators.length === 3) {
        setHomePriceConsumption(monthlyForecast.totalValue)
      } else {
        let totalPrice = 0
        performanceIndicators.forEach((fluid, index) => {
          if (monthlyForecast.fluidForecast[index].value)
            totalPrice += monthlyForecast.fluidForecast[index].value
        })
        setHomePriceConsumption(totalPrice)
      }
    },
    [performanceIndicators]
  )

  useEffect(() => {
    let subscribed = true
    async function loadAverageConsumption() {
      const profileTypeEntityService = new ProfileTypeEntityService(client)
      const profileType: ProfileType | null =
        await profileTypeEntityService.getProfileType(
          analysisMonth.minus({ month: 1 }).startOf('month')
        )
      if (profileType !== null) {
        const profileTypeService: ProfileTypeService = new ProfileTypeService(
          profileType,
          client,
          analysisMonth.minus({ month: 1 }).year
        )
        const monthlyForecast: MonthlyForecast =
          await profileTypeService.getMonthlyForecast(
            analysisMonth.minus({ month: 1 }).startOf('month').month
          )
        if (subscribed) {
          setForecast(monthlyForecast)
          if (monthlyForecast) {
            getTotalValueWithConnectedFluids(monthlyForecast)
          }
          setIsLoading(false)
        }
      }
    }
    loadAverageConsumption()
    return () => {
      subscribed = false
    }
  }, [
    profile.monthlyAnalysisDate.month,
    getTotalValueWithConnectedFluids,
    client,
    analysisMonth.month,
    analysisMonth,
  ])

  const profileNotCompleted = (
    <div className="no-profile">
      <div className="text-16-normal">
        {t('analysis.approximative_description')}
      </div>
      <Button
        aria-label={t('analysis.accessibility.button_go_to_profil')}
        onClick={() => navigate('/profileType')}
        classes={{
          root: 'btn-highlight',
          label: 'text-18-bold',
        }}
      >
        {t('analysis.accessibility.button_go_to_profil')}
      </Button>
      <Icon
        icon={PlaceHolderIcon}
        width="100%"
        height="60%"
        alt="pas de profil remplis"
      />
    </div>
  )

  const Consumption = (
    <div className="analysis-graph">
      {isLoading ? (
        <div className="loader-container">
          <Loader />
        </div>
      ) : (
        <>
          <div className="consumption-title text-20-bold">
            <div className="user-title">{t('analysis.user_consumption')}</div>
            <div />
            <div className="average-title">{t(`analysis.comparison`)}</div>
          </div>
          <ProfileComparatorRow
            fluid={FluidType.MULTIFLUID}
            userPriceConsumption={userPriceConsumption}
            homePriceConsumption={homePriceConsumption}
            performanceValue={null}
            forecast={forecast}
            connected={performanceIndicators.length > 0}
            noData={false}
          />
          {performanceIndicators.map(
            (indicator, index) =>
              indicator.value && (
                <ProfileComparatorRow
                  key={index}
                  fluid={index}
                  userPriceConsumption={userPriceConsumption}
                  homePriceConsumption={homePriceConsumption}
                  performanceValue={indicator.value}
                  forecast={forecast}
                  connected={true}
                  noData={false}
                />
              )
          )}
          {emptyFluidTypes.length > 0 && <hr className="consumption-sep" />}
          {emptyFluidTypes.map(fluid => (
            <ProfileComparatorRow
              key={fluid}
              fluid={fluid}
              userPriceConsumption={userPriceConsumption}
              homePriceConsumption={homePriceConsumption}
              performanceValue={null}
              forecast={forecast}
              connected={false}
              noData={true}
            />
          ))}

          <Accordion
            expanded={activeAverageHome}
            onChange={toggleAccordion}
            classes={{
              root: 'expansion-panel-root',
            }}
          >
            <AccordionSummary
              aria-label={t(
                'profile_type.accessibility.button_toggle_average_home'
              )}
              expandIcon={
                <Icon icon={chevronDown} size={16} className="accordion-icon" />
              }
              classes={{
                root: 'expansion-panel-summary',
                content: 'expansion-panel-content',
              }}
            >
              <div className="accordion-title accordion-title">
                {t('analysis.average_home')}
              </div>
            </AccordionSummary>
            <AccordionDetails
              classes={{
                root: 'expansion-panel-details',
              }}
            >
              <span className="accordion-desc text-16-normal">
                {t('analysis.average_home_description')}
              </span>
            </AccordionDetails>
          </Accordion>
        </>
      )}
    </div>
  )

  return (
    <>
      <div className="status-header">
        <div>
          <StyledIcon icon={AnalysisIcon} size={44} />
          <p className="text-16-normal">{t('analysis.compare.title')}</p>
        </div>
        {profile.isProfileTypeCompleted && (
          <Button
            aria-label={t('analysis.accessibility.button_go_to_profil')}
            onClick={() => navigate('/profileType')}
            classes={{
              root: 'btn-secondary-negative',
              label: 'text-16-normal',
            }}
          >
            <StyledIcon icon={ProfileEditIcon} size={40} />
          </Button>
        )}
      </div>

      <div className="analysis-graph">
        {!profile.isProfileTypeCompleted && profileNotCompleted}
        {profile.isProfileTypeCompleted && Consumption}
      </div>
    </>
  )
}

export default ProfileComparator
