import Button from '@material-ui/core/Button'
import * as Sentry from '@sentry/react'
import TrailingIcon from 'assets/icons/ico/trailing-icon.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import useKonnectorAuth from 'components/Hooks/useKonnectorAuth'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType } from 'enums'
import { Account, AccountAuthData } from 'models'
import React, { useEffect, useState } from 'react'
import { useAppSelector } from 'store/hooks'
import logApp from 'utils/logger'
import { getPartnerPicto } from 'utils/picto'
import './formLogin.scss'

const FormLogin = () => {
  const { t } = useI18n()
  const { fluidStatus, lastEpglLogin } = useAppSelector(
    state => state.ecolyo.global
  )
  const currentFluidStatus = fluidStatus[FluidType.WATER]
  const konnectorSlug: string =
    currentFluidStatus.connection.konnectorConfig.slug
  const fluidName: string = FluidType[currentFluidStatus.fluidType]
  const account: Account | null = currentFluidStatus.connection.account

  const [login, setLogin] = useState<string>('')
  const [password, setPassword] = useState<string>('')
  const [error, setError] = useState<string>('')
  const [loading, setLoading] = useState<boolean>(false)

  const icon = getPartnerPicto(konnectorSlug)
  const [connect, update, connectError] = useKonnectorAuth(
    currentFluidStatus.fluidType,
    login,
    password
  )

  const changeLogin = (value: string) => {
    if ((/\d/.test(value) && value.length <= 7) || value === '') {
      setError('')
      setLogin(value)
    }
  }

  const changePassword = (value: string) => {
    setError('')
    setPassword(value)
  }

  const revealPassword = (idInput: string) => {
    const input = document.getElementById(idInput)
    if (input) {
      if (input.getAttribute('type') === 'password') {
        input.setAttribute('type', 'text')
      } else {
        input.setAttribute('type', 'password')
      }
    }
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault()
    try {
      setError('')
      setLoading(true)
      if (!login || !password) {
        setError(t('konnector_form.error_no_login_password'))
        setLoading(false)
        return null
      }
      if (!account) {
        await connect()
        setLoading(false)
      } else {
        await update()
      }
    } catch (error) {
      logApp.error(error)
      Sentry.captureException(error)
      setLoading(false)
    }
  }

  useEffect(() => {
    if (connectError) setError(connectError)
  }, [connectError])

  useEffect(() => {
    if (account?.auth) {
      const auth = account.auth
      const authData = auth as AccountAuthData
      if (authData.login) {
        setLogin(authData.login)
      }
    } else if (lastEpglLogin) {
      setLogin(lastEpglLogin)
      setError(t('konnector_form.error_login_failed'))
    }
  }, [account, lastEpglLogin, t])

  return (
    <form
      className="form"
      onSubmit={(e: React.FormEvent<HTMLFormElement>) => handleSubmit(e)}
    >
      <div className="form-group">
        <input
          id={'idFieldLogin' + fluidName}
          type="number"
          className={
            error === ''
              ? 'form-control form-input'
              : 'form-control form-input --error'
          }
          aria-label={t(`auth.${konnectorSlug}.connect_form.login`)}
          placeholder={t(`auth.${konnectorSlug}.connect_form.login`)}
          name="login"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            changeLogin(e.target.value)
          }
          value={login}
          inputMode="numeric"
        />
      </div>
      <div className="form-group">
        <input
          id={'idFieldPassword' + fluidName}
          type="password"
          className={
            error === ''
              ? 'form-control form-input'
              : 'form-control form-input --error'
          }
          aria-label={t(`auth.${konnectorSlug}.connect_form.password`)}
          placeholder={t(`auth.${konnectorSlug}.connect_form.password`)}
          name="password"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            changePassword(e.target.value)
          }
          value={password}
        />
        <span>
          <StyledIconButton
            icon={TrailingIcon}
            aria-label={t('auth.accessibility.button_reveal_password')}
            className="form-trailing-icon"
            sized={22}
            onClick={() => revealPassword('idFieldPassword' + fluidName)}
          />
        </span>
      </div>
      <div className="form-message">{error === '' ? null : error}</div>
      <div className="connection-form-connect-button">
        <Button
          type="submit"
          aria-label={t('auth.accessibility.button_connect')}
          disabled={loading}
          classes={{
            root: 'btn-highlight',
            label: 'text-18-bold',
          }}
        >
          <div className="connection-form-connect-button-content">
            <div className="connection-form-connect-button-content-icon">
              {icon && <StyledIcon icon={icon} size={80} />}
            </div>
            <div className="connection-form-connect-button-text text-18-bold">
              <div>{t(`auth.${konnectorSlug}.connect_form.label`)}</div>
            </div>
          </div>
        </Button>
      </div>
    </form>
  )
}

export default FormLogin
