import Loader from 'components/Loader/Loader'
import { FluidState, FluidType, TimeStep } from 'enums'
import { mount } from 'enzyme'
import React from 'react'
import { Provider } from 'react-redux'
import * as chartActions from 'store/chart/chart.slice'
import { mockCustomPopup } from 'tests/__mocks__/customPopup.mock'
import {
  mockExpiredElec,
  mockExpiredGas,
} from 'tests/__mocks__/fluidStatusData.mock'
import { mockTestProfile1 } from 'tests/__mocks__/profileType.mock'
import {
  createMockEcolyoStore,
  mockChartState,
  mockInitialEcolyoState,
  mockModalState,
} from 'tests/__mocks__/store'
import ConsumptionView from './ConsumptionView'

const mockUpdateProfile = jest.fn()
jest.mock('services/profile.service', () => {
  return jest.fn(() => ({
    updateProfile: mockUpdateProfile,
  }))
})

jest.mock('components/Header/CozyBar', () => 'mock-cozybar')
jest.mock('components/Header/Header', () => 'mock-header')
jest.mock('components/DateNavigator/DateNavigator', () => 'mock-dateNavigator')
jest.mock('components/Content/Content', () => 'mock-content')
jest.mock(
  'components/Consumption/FluidButtons/FluidButtons',
  () => 'mock-fluidButtons'
)
jest.mock('components/FluidChart/FluidChart', () => 'mock-fluidchart')
jest.mock(
  'components/Consumption/ConsumptionDetails/ConsumptionDetails',
  () => 'mock-consumptionDetails'
)
jest.mock('components/Connection/Connection', () => 'mock-connection')
jest.mock(
  'components/Konnector/KonnectorViewerCard',
  () => 'mock-konnectorViewerCard'
)
jest.mock(
  'components/PartnerIssue/PartnerIssueModal',
  () => 'mock-partnerIssueModal'
)
jest.mock('components/CustomPopup/CustomPopupModal', () => 'mock-customPopup')
jest.mock(
  'components/ReleaseNotesModal/ReleaseNotesModal',
  () => 'mock-releaseNotes'
)
jest.mock(
  'components/Connection/SGEConnect/SgeConnectView',
  () => 'mock-SgeConnectView'
)

const setCurrentTimeStepSpy = jest.spyOn(chartActions, 'setCurrentTimeStep')

const mockFluidStatus = mockInitialEcolyoState.global.fluidStatus
mockFluidStatus[FluidType.ELECTRICITY].status = FluidState.DONE

const mockChartStateShowOffline = { ...mockChartState, showOfflineData: true }

describe('ConsumptionView component', () => {
  it('should be rendered correctly', async () => {
    const store = createMockEcolyoStore({
      chart: {
        ...mockChartState,
        loading: false,
        showOfflineData: true,
      },
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-cozybar').exists()).toBeTruthy()
    expect(wrapper.find('mock-header').exists()).toBeTruthy()
    expect(wrapper.find('mock-dateNavigator').exists()).toBeTruthy()
    expect(wrapper.find('mock-fluidButtons').exists()).toBeTruthy()
    expect(wrapper.find('mock-fluidchart').exists()).toBeTruthy()
    expect(wrapper.find('mock-consumptionDetails').exists()).toBeTruthy()
  })

  it('should display a spinner when fluid connected and is loading', () => {
    const store = createMockEcolyoStore({
      chart: {
        ...mockChartState,
        loading: true,
        showOfflineData: true,
      },
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-cozybar').exists()).toBeTruthy()
    expect(wrapper.find('mock-header').exists()).toBeTruthy()
    expect(wrapper.find('mock-dateNavigator').exists()).toBeTruthy()
    expect(wrapper.find(Loader).exists()).toBeTruthy()
  })

  it('should set CurrentTimeStep to WEEK when fluid != ELECTRICITY and timeStep = HALF_AN_HOUR', () => {
    const store = createMockEcolyoStore({
      chart: {
        ...mockChartState,
        currentTimeStep: TimeStep.HALF_AN_HOUR,
      },
      global: {
        fluidStatus: mockInitialEcolyoState.global.fluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.GAS} />
      </Provider>
    )
    expect(setCurrentTimeStepSpy).toHaveBeenCalledTimes(1)
    expect(setCurrentTimeStepSpy).toHaveBeenCalledWith(TimeStep.WEEK)
  })

  it('should render konnector list when no fluid is connected', () => {
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: [],
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.MULTIFLUID} />
      </Provider>
    )
    expect(wrapper.find('mock-consumptionDetails').exists()).toBeTruthy()
  })

  it('should render mutlifluid consumption if at least one fluid is connected', () => {
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.MULTIFLUID} />
      </Provider>
    )
    expect(wrapper.find('.consumptionview-content').exists()).toBeTruthy()
  })

  it('should render Electricity when elec is connected', () => {
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('.consumptionview-content').exists()).toBeTruthy()
    expect(wrapper.find('mock-consumptionDetails').exists()).toBeTruthy()
  })

  // todo describe and add multiple fluids ?
  it('should render partner issue Modal', async () => {
    const updatedStatus = mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredElec
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: { partnersIssueModal: { enedis: false, grdf: false, egl: true } },
    })
    mockUpdateProfile.mockResolvedValue(mockTestProfile1)
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-partnerIssueModal').exists()).toBeTruthy()
  })
  it('should show expired modal when a GRDF consent is expired', () => {
    const updatedStatus = mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredGas
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.GAS} />
      </Provider>
    )
    expect(wrapper.find('.title').text()).toBe('consent_outdated.title.2')
  })
  it('should show expired modal when a Enedis consent is expired', () => {
    const updatedStatus = mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredElec
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: mockModalState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )

    expect(wrapper.find('.title').text()).toBe('consent_outdated.title.0')
  })
  it('should render customPopup Modal', async () => {
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      modal: {
        ...mockModalState,
        customPopupModal: mockCustomPopup,
      },
    })
    mockUpdateProfile.mockResolvedValue(mockTestProfile1)
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-customPopup').exists()).toBeTruthy()
  })
  it('should render releaseNotesModal if releaseNotes.show is true', async () => {
    const store = createMockEcolyoStore({
      chart: mockChartStateShowOffline,
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: {
          show: true,
          notes: [{ description: 'description', title: 'title' }],
        },
      },
      modal: mockModalState,
    })
    mockUpdateProfile.mockResolvedValue(mockTestProfile1)
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-releaseNotes').exists()).toBeTruthy()
  })
})
