import { Grow, ListItemIcon, Menu, MenuItem } from '@material-ui/core/'
import Button from '@material-ui/core/Button'
import CheckIcon from 'assets/icons/ico/check.svg'
import SortIcon from 'assets/icons/ico/sort.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import EcogestureCard from 'components/Ecogesture/EcogestureCard/EcogestureCard'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { Usage } from 'enums'
import { Ecogesture } from 'models'
import React, { useState } from 'react'
import { useNavigate } from 'react-router-dom'
import { updateEcogestureFilter } from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import './ecogestureList.scss'

interface EcogestureListProps {
  list: Ecogesture[]
  displaySelection: boolean
  selectionTotal: number
  selectionViewed: number
  handleReinitClick?: () => void
}

const EcogestureList = ({
  list,
  displaySelection,
  selectionTotal,
  selectionViewed,
  handleReinitClick,
}: EcogestureListProps) => {
  const { t } = useI18n()
  const navigate = useNavigate()
  const dispatch = useAppDispatch()
  const { ecogestureFilter } = useAppSelector(state => state.ecolyo.global)
  const [openDropDown, setOpenDropDown] = useState<boolean>(false)
  const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null)

  const toggleDropDown = () => {
    setOpenDropDown(prev => !prev)
  }

  const toggleMenu = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget)
  }

  const closeMenu = (usage?: Usage) => {
    usage !== undefined && dispatch(updateEcogestureFilter(usage))
    setAnchorEl(null)
  }

  const renderEcogestureContent = () => {
    const isAllUsage = ecogestureFilter === Usage.ALL
    const filteredEcogestures = list.filter(ecogesture =>
      isAllUsage ? true : ecogesture.usage === ecogestureFilter
    )

    if (filteredEcogestures.length > 0) {
      return filteredEcogestures.map(ecogesture => (
        <EcogestureCard key={ecogesture.id} ecogesture={ecogesture} />
      ))
    }

    return (
      <div className="ec-filter-error">
        <div className="text-20-normal">
          {t('ecogesture.no_ecogesture_filter.text1')}
        </div>
        <div className="text-16-italic">
          {t('ecogesture.no_ecogesture_filter.text2')}
        </div>
      </div>
    )
  }

  const selectFilters = () => (
    <div className="filters text-16-normal">
      <div
        className="filter-button"
        onClick={() => toggleDropDown()}
        tabIndex={0}
        onBlur={e => {
          if (e.relatedTarget === null) toggleDropDown()
        }}
      >
        <Button
          classes={{
            root: 'btn-secondary-negative',
            label: 'text-14-normal',
          }}
          aria-controls="simple-menu"
          aria-haspopup="true"
          aria-label={t(`ecogesture.MENU_TITLE`)}
          onClick={toggleMenu}
          variant="contained"
        >
          <StyledIcon icon={SortIcon} size={20} />
          <span className={openDropDown ? 'ecogestures opened' : 'ecogestures'}>
            {ecogestureFilter === Usage.ALL
              ? t(`ecogesture.MENU_TITLE`)
              : t(`ecogesture.${Usage[ecogestureFilter]}`)}
          </span>
        </Button>
        <Menu
          anchorEl={anchorEl}
          keepMounted
          open={Boolean(anchorEl)}
          onClose={() => closeMenu()}
          PopoverClasses={{
            paper: 'filter-menu',
          }}
          variant="menu"
          MenuListProps={{ className: 'filter-menu-list' }}
        >
          {Object.values(Usage).map((usage, key) => {
            const active = usage === ecogestureFilter
            return (
              typeof usage === 'number' && (
                <MenuItem
                  classes={{
                    root: `${active ? 'item-active' : ''}`,
                  }}
                  key={key}
                  selected={active}
                  onClick={() => closeMenu(usage)}
                >
                  {t(`ecogesture.${Usage[usage]}`)}
                  {active && (
                    <ListItemIcon classes={{ root: 'filter-menu-icon' }}>
                      <StyledIcon icon={CheckIcon} size={13} />
                    </ListItemIcon>
                  )}
                </MenuItem>
              )
            )
          })}
        </Menu>
      </div>
    </div>
  )

  const continueSelection = () => (
    <Grow in={displaySelection}>
      <div className="selection text-16-normal">
        <span>{t('ecogesture.selection')}</span>
        <span>
          {`(${selectionViewed} ${t(
            'ecogesture.selection_2'
          )} ${selectionTotal})`}
        </span>
        <Button
          aria-label={t('ecogesture.accessibility.button_selection')}
          classes={{
            root: 'btn-highlight',
            label: 'text-16-bold',
          }}
          onClick={() => navigate('/ecogesture-selection')}
        >
          {t('ecogesture.button_selection')}
        </Button>
      </div>
    </Grow>
  )

  return (
    <div className="ecogesture-root">
      <div className="efficiency-button-content">
        {selectFilters()}
        {displaySelection && continueSelection()}
      </div>

      <div className="ecogesture-content">
        {renderEcogestureContent()}
        {!displaySelection && handleReinitClick && (
          <Button
            onClick={handleReinitClick}
            classes={{
              root: 'btn-secondary-negative',
              label: 'text-16-normal',
            }}
          >
            {t('ecogesture.reinit')}
          </Button>
        )}
      </div>
    </div>
  )
}

export default EcogestureList
