import { Button } from '@material-ui/core'
import TimeStepSelector from 'components/FluidChart/TimeStepSelector/TimeStepSelector'
import { FluidType, TimeStep } from 'enums'
import { mount } from 'enzyme'
import toJson from 'enzyme-to-json'
import { DateTime } from 'luxon'
import React from 'react'
import { Provider } from 'react-redux'
import UsageEventService from 'services/usageEvent.service'
import * as chartActions from 'store/chart/chart.slice'
import { createMockEcolyoStore } from 'tests/__mocks__/store'
import { waitForComponentToPaint } from 'tests/__mocks__/testUtils'

jest.mock('services/usageEvent.service')
const mockAddEvent = jest.fn()
UsageEventService.addEvent = mockAddEvent

const setCurrentTimeStepSpy = jest.spyOn(chartActions, 'setCurrentTimeStep')
const setCurrentIndexSpy = jest.spyOn(chartActions, 'setCurrentIndex')
const setSelectedDateSpy = jest.spyOn(chartActions, 'setSelectedDate')

describe('TimeStepSelector component', () => {
  beforeEach(() => {
    setCurrentTimeStepSpy.mockClear()
  })

  it('should render component properly with 4 timesteps', async () => {
    const store = createMockEcolyoStore({
      chart: {
        currentTimeStep: TimeStep.DAY,
        selectedDate: DateTime.local().setZone('utc', {
          keepLocalTime: true,
        }),
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.WATER} />
      </Provider>
    )
    expect(wrapper.find('.circle').at(3).exists()).toBeTruthy()
    expect(wrapper.find('.circle').at(4).exists()).toBeFalsy()
    await waitForComponentToPaint(wrapper)
    expect(toJson(wrapper)).toMatchSnapshot()
  })

  it('should render component properly with 5 timesteps', () => {
    const store = createMockEcolyoStore({
      chart: {
        currentTimeStep: TimeStep.DAY,
        selectedDate: DateTime.local().setZone('utc', {
          keepLocalTime: true,
        }),
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('.circle').at(4).exists()).toBeTruthy()
    expect(wrapper.find('.circle').at(5).exists()).toBeFalsy()
  })

  it('should define next TimeStep and dispatch it', () => {
    const store = createMockEcolyoStore({
      chart: {
        currentTimeStep: TimeStep.YEAR,
        selectedDate: DateTime.local().setZone('utc', {
          keepLocalTime: true,
        }),
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.WATER} />
      </Provider>
    )
    wrapper.find('#day').first().simulate('click')
    expect(setCurrentTimeStepSpy).toHaveBeenCalledTimes(1)
    expect(setCurrentTimeStepSpy).toHaveBeenCalledWith(TimeStep.DAY)
    expect(setCurrentIndexSpy).toHaveBeenCalledTimes(1)
  })
  it('should go to todays day with timestep week', () => {
    const store = createMockEcolyoStore({
      chart: {
        currentTimeStep: TimeStep.YEAR,
        selectedDate: DateTime.local().setZone('utc', {
          keepLocalTime: true,
        }),
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.WATER} />
      </Provider>
    )
    wrapper.find(Button).first().simulate('click')
    expect(setCurrentTimeStepSpy).toHaveBeenCalledTimes(1)
    expect(setCurrentTimeStepSpy).toHaveBeenCalledWith(TimeStep.WEEK)
    expect(setCurrentIndexSpy).toHaveBeenCalledTimes(2)
    expect(setSelectedDateSpy).toHaveBeenCalledTimes(1)
  })
})
