import CozyBar from 'components/Header/CozyBar'
import { ScreenType } from 'enums'
import { mount } from 'enzyme'
import toJson from 'enzyme-to-json'
import React from 'react'
import { Provider } from 'react-redux'
import * as ModalAction from 'store/modal/modal.slice'
import { createMockEcolyoStore } from 'tests/__mocks__/store'

interface BarProps {
  children: React.ReactNode
}

jest.mock('utils/cozyBar', () => {
  return {
    BarLeft: ({ children }: BarProps) => children,
    BarCenter: ({ children }: BarProps) => children,
    BarRight: ({ children }: BarProps) => children,
  }
})

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

describe('CozyBar component', () => {
  const store = createMockEcolyoStore()
  it('should be rendered correctly without without left arrow', () => {
    const wrapper = mount(
      <Provider store={store}>
        <CozyBar />
      </Provider>
    )
    expect(toJson(wrapper)).toMatchSnapshot()
  })

  describe('should test displayBackArrow', () => {
    it('should call navigate(-1) when back button is clicked and no function is provided', () => {
      const wrapper = mount(
        <Provider store={store}>
          <CozyBar displayBackArrow={true} />
        </Provider>
      )
      expect(toJson(wrapper)).toMatchSnapshot()
      expect(wrapper.find('BarLeft')).toHaveLength(1)
      wrapper.find('BarLeft').find('.cv-button').first().simulate('click')
      expect(mockedNavigate).toHaveBeenCalled()
    })

    it('should call backFunction() when back button is clicked and function is provided', () => {
      const mockBackFunction = jest.fn()
      const wrapper = mount(
        <Provider store={store}>
          <CozyBar displayBackArrow={true} backFunction={mockBackFunction} />
        </Provider>
      )
      expect(toJson(wrapper)).toMatchSnapshot()
      expect(wrapper.find('BarLeft')).toHaveLength(1)
      wrapper.find('BarLeft').find('.cv-button').first().simulate('click')
      expect(mockBackFunction).toHaveBeenCalled()
    })
  })

  it('should call handleClickFeedbacks when feedback button is clicked', () => {
    const wrapper = mount(
      <Provider store={store}>
        <CozyBar />
      </Provider>
    )
    const updateModalSpy = jest.spyOn(ModalAction, 'openFeedbackModal')
    wrapper.find('BarRight').find('.cv-button').first().simulate('click')
    expect(updateModalSpy).toHaveBeenCalledWith(true)
  })

  it('should not be rendered with screen type different from mobile', () => {
    const store = createMockEcolyoStore({
      global: { screenType: ScreenType.DESKTOP },
    })
    const wrapper = mount(
      <Provider store={store}>
        <CozyBar />
      </Provider>
    )
    // Snapshot should be empty
    expect(toJson(wrapper)).toMatchSnapshot()
  })
})
