import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
  Button,
} from '@material-ui/core'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import ErrorNotif from 'assets/icons/ico/notif_error.svg'
import PartnersIssueNotif from 'assets/icons/ico/notif_maintenance.svg'
import WarningNotif from 'assets/icons/ico/notif_warning.svg'
import OfflinePicto from 'assets/icons/visu/offline-param.svg'
import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Connection from 'components/Connection/Connection'
import ConnectionNotFound from 'components/Konnector/ConnectionNotFound/ConnectionNotFound'
import ConnectionResult from 'components/Konnector/ConnectionResult/ConnectionResult'
import KonnectorModal from 'components/Konnector/KonnectorModal'
import { useClient } from 'cozy-client'
import { isKonnectorRunning } from 'cozy-harvest-lib/dist/helpers/triggers'
import ConnectionFlow from 'cozy-harvest-lib/dist/models/ConnectionFlow'
import {
  ERROR_EVENT,
  LOGIN_SUCCESS_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/flowEvents'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import {
  FluidSlugType,
  FluidState,
  FluidType,
  KonnectorError,
  UsageEventType,
  UserChallengeState,
  UserDuelState,
} from 'enums'
import { DateTime } from 'luxon'
import {
  Account,
  AccountAuthData,
  FluidConnection,
  FluidStatus,
  Konnector,
  PartnersInfo,
  Trigger,
  UsageEvent,
} from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { useNavigate } from 'react-router-dom'
import AccountService from 'services/account.service'
import ChallengeService from 'services/challenge.service'
import DateChartService from 'services/dateChart.service'
import FluidService from 'services/fluid.service'
import PartnersInfoService from 'services/partnersInfo.service'
import UsageEventService from 'services/usageEvent.service'
import { setChallengeConsumption } from 'store/challenge/challenge.slice'
import { setSelectedDate, setShowOfflineData } from 'store/chart/chart.slice'
import {
  setFluidStatus,
  setLastEpglLogin,
  toggleChallengeDuelNotification,
  updateFluidConnection,
} from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import { openConnectionModal } from 'store/modal/modal.slice'
import { getParamPicto } from 'utils/picto'
import { getKonnectorSlug } from 'utils/utils'
import './konnectorViewerCard.scss'

interface KonnectorViewerCardProps {
  isDisconnected: boolean
  showOfflineData: boolean
  active: boolean
  fluidType: FluidType
  setActive: React.Dispatch<React.SetStateAction<boolean>>
}

const KonnectorViewerCard = ({
  isDisconnected,
  showOfflineData,
  active,
  fluidType,
  setActive,
}: KonnectorViewerCardProps) => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useAppDispatch()
  const navigate = useNavigate()
  const {
    challenge: { currentChallenge },
    global: { fluidStatus, shouldRefreshConsent, partnersInfo },
  } = useAppSelector(state => state.ecolyo)
  const currentFluidStatus = fluidStatus[fluidType]
  const fluidSlug = currentFluidStatus.connection.konnectorConfig.slug
  const fluidState = currentFluidStatus.status
  const konnector: Konnector | null = currentFluidStatus.connection.konnector
  const account: Account | null = currentFluidStatus.connection.account
  const trigger: Trigger | null = currentFluidStatus.connection.trigger

  const [openModal, setOpenModal] = useState(false)
  const [isUpdating, setIsUpdating] = useState(false)
  const [isLogging, setIsLogging] = useState(fluidSlug !== FluidSlugType.GAS)
  const [konnectorErrorDescription, setKonnectorErrorDescription] =
    useState<KonnectorError | null>(null)
  const [konnectorState, setKonnectorState] = useState<string | null>(null)
  const [isOutdatedData, setIsOutdatedData] = useState<number | null>(null)
  const fluidService = useMemo(() => new FluidService(client), [client])
  const partnersInfoService = useMemo(
    () => new PartnersInfoService(client),
    [client]
  )

  const lastDataDate =
    fluidType !== FluidType.MULTIFLUID && currentFluidStatus.lastDataDate
      ? currentFluidStatus.lastDataDate.toLocaleString() + fluidType
      : fluidType

  const iconType = getParamPicto(currentFluidStatus.fluidType)

  const toggleAccordion = () => {
    setActive(prev => !prev)
  }

  const updateGlobalFluidStatus = useCallback(async (): Promise<
    FluidStatus[]
  > => {
    const _updatedFluidStatus: FluidStatus[] =
      await fluidService.getFluidStatus()
    const refDate = DateTime.fromISO('0001-01-01')
    let _lastDataDate: DateTime | null = DateTime.fromISO('0001-01-01')
    for (const fluid of _updatedFluidStatus) {
      if (fluid?.lastDataDate && fluid?.lastDataDate > _lastDataDate) {
        _lastDataDate = fluid.lastDataDate
      }
    }
    if (_lastDataDate > refDate) {
      dispatch(setSelectedDate(_lastDataDate))
    }

    return _updatedFluidStatus
  }, [dispatch, fluidService])

  const refreshChallengeState = useCallback(async () => {
    if (
      currentChallenge?.state === UserChallengeState.DUEL &&
      currentChallenge?.duel.state === UserDuelState.ONGOING
    ) {
      const challengeService = new ChallengeService(client)
      const { updatedUserChallenge, dataloads } =
        await challengeService.initChallengeDuelProgress(currentChallenge)
      dispatch(
        setChallengeConsumption({
          userChallenge: updatedUserChallenge,
          currentDataload: dataloads,
        })
      )
      // Check is duel is done and display notification
      const { isDone } = await challengeService.isChallengeDone(
        updatedUserChallenge,
        dataloads
      )
      dispatch(toggleChallengeDuelNotification(isDone))
    }
  }, [client, dispatch, currentChallenge])

  const handleAccountDeletion = useCallback(async () => {
    await refreshChallengeState()
    const _updatedFluidStatus = await updateGlobalFluidStatus()
    if (_updatedFluidStatus.length > 0) {
      const partnersInfo: PartnersInfo | undefined =
        await partnersInfoService.getPartnersInfo()
      const updatedFluidStatus: FluidStatus[] =
        await fluidService.getFluidStatus(partnersInfo)
      dispatch(setFluidStatus(updatedFluidStatus))
    }
    setActive(false)
  }, [
    refreshChallengeState,
    updateGlobalFluidStatus,
    setActive,
    partnersInfoService,
    fluidService,
    dispatch,
  ])

  const handleConnectionEnd = useCallback(
    async (isSuccess?: boolean) => {
      // CASE FOR GLOBAL LOGIN FAILED
      const isGlobalLoginFailed =
        konnectorErrorDescription === KonnectorError.LOGIN_FAILED ||
        konnectorErrorDescription === KonnectorError.UNKNOWN_ERROR ||
        konnectorErrorDescription === KonnectorError.CHALLENGE_ASKED ||
        konnectorErrorDescription === KonnectorError.CRITICAL ||
        konnectorErrorDescription === KonnectorError.MISSING_SECRET

      // CASE FOR ENEDIS CODE INSEE ERROR
      const isEnedisCodeInseeError =
        !isUpdating && fluidType === FluidType.ELECTRICITY

      const shouldDeleteAccount =
        account &&
        !isSuccess &&
        currentFluidStatus &&
        currentFluidStatus.connection.account &&
        (isGlobalLoginFailed || isEnedisCodeInseeError)

      if (shouldDeleteAccount) {
        // KEEP LAST LOGIN FOR EPGL
        let lastEpglLogin = ''
        if (
          fluidSlug === FluidSlugType.WATER &&
          currentFluidStatus.connection.account?.auth
        ) {
          const auth = currentFluidStatus.connection.account
            .auth as AccountAuthData
          lastEpglLogin = auth.login
        }
        // DELETE ACCOUNT
        const accountService = new AccountService(client)
        await accountService.deleteAccount(account)
        await handleAccountDeletion()

        // RESTORE LAST KNOWN CREDENTIALS
        if (lastEpglLogin) {
          dispatch(setLastEpglLogin(lastEpglLogin))
        }
      } else {
        if (isSuccess && currentFluidStatus.lastDataDate === null) {
          // UPDATE THE DACC EVENT STATUS TO SUCCESS
          await UsageEventService.updateConnectionAttemptEvent(
            client,
            fluidSlug
          )
        }
        // const partnersInfo = await partnersInfoService.getPartnersInfo()
        const updatedFluidStatus = await fluidService.getFluidStatus(
          partnersInfo
        )
        dispatch(setFluidStatus(updatedFluidStatus))
      }

      setActive(false)
      setOpenModal(false)
      // TODO null state seems to be read before modal closing and display a success icon in modal
      setKonnectorState(null)
      setKonnectorErrorDescription(null)
    },
    [
      account,
      konnectorErrorDescription,
      currentFluidStatus,
      isUpdating,
      fluidType,
      setActive,
      fluidSlug,
      client,
      handleAccountDeletion,
      fluidService,
      partnersInfo,
      dispatch,
    ]
  )

  const sendUsageEventSuccess = useCallback(
    async (
      konnectorSlug: string,
      isConnexion: boolean
    ): Promise<UsageEvent> => {
      return UsageEventService.addEvent(client, {
        type: isConnexion
          ? UsageEventType.KONNECTOR_CONNECT_EVENT
          : UsageEventType.KONNECTOR_REFRESH_EVENT,
        target: konnectorSlug,
        result: 'success',
      })
    },
    [client]
  )

  const sendUsageEventError = useCallback(
    async (
      konnectorSlug: string,
      isConnexion: boolean
    ): Promise<UsageEvent> => {
      return UsageEventService.addEvent(client, {
        type: isConnexion
          ? UsageEventType.KONNECTOR_CONNECT_EVENT
          : UsageEventType.KONNECTOR_REFRESH_EVENT,
        target: konnectorSlug,
        result: 'error',
      })
    },
    [client]
  )

  const toggleModalConnection = useCallback(() => {
    switch (fluidType) {
      case FluidType.ELECTRICITY:
        navigate('/sge-connect')
        break
      case FluidType.GAS:
      case FluidType.WATER:
        dispatch(setShowOfflineData(false))
        dispatch(openConnectionModal(true))
        break
    }
  }, [dispatch, fluidType, navigate])

  const getConnectionCard = useCallback(() => {
    if (showOfflineData && !account) {
      return (
        <Button
          classes={{
            root: 'btn-highlight',
            label: 'text-16-bold',
          }}
          onClick={toggleModalConnection}
        >
          {t(`auth.${getKonnectorSlug(fluidType)}.connect`)}
        </Button>
      )
    }
    if (fluidState === FluidState.KONNECTOR_NOT_FOUND && !isUpdating) {
      return <ConnectionNotFound konnectorSlug={fluidSlug} />
    }
    // Handle login failed for EGL
    else if (
      (fluidState === FluidState.ERROR_LOGIN_FAILED &&
        fluidType === FluidType.WATER) ||
      (account && currentFluidStatus.status !== FluidState.NOT_CONNECTED)
    ) {
      return (
        <ConnectionResult
          handleAccountDeletion={handleAccountDeletion}
          fluidType={fluidType}
          key={lastDataDate}
        />
      )
    } else {
      return <Connection fluidType={currentFluidStatus.fluidType} />
    }
  }, [
    account,
    currentFluidStatus.fluidType,
    currentFluidStatus.status,
    fluidSlug,
    fluidState,
    fluidType,
    handleAccountDeletion,
    isUpdating,
    lastDataDate,
    showOfflineData,
    t,
    toggleModalConnection,
  ])

  const callbackResponse = useCallback(
    async (_state: string) => {
      if (_state !== LOGIN_SUCCESS_EVENT) {
        const updatedConnection: FluidConnection = {
          ...currentFluidStatus.connection,
          shouldLaunchKonnector: false,
        }
        dispatch(
          updateFluidConnection({
            fluidType: currentFluidStatus.fluidType,
            fluidConnection: updatedConnection,
          })
        )
        Promise.all([
          await refreshChallengeState(),
          await updateGlobalFluidStatus(),
        ])
        setKonnectorState(_state)
      }
    },
    [
      dispatch,
      currentFluidStatus.connection,
      currentFluidStatus.fluidType,
      refreshChallengeState,
      updateGlobalFluidStatus,
    ]
  )

  const getIconForStatus = (
    status: FluidState,
    maintenance: boolean,
    connection: FluidConnection,
    outdatedData: number | null
  ) => {
    if (maintenance) {
      return (
        <StyledIcon
          icon={PartnersIssueNotif}
          size={24}
          className="konnector-state-picto"
        />
      )
    }

    if (
      (status === FluidState.ERROR ||
        status === FluidState.ERROR_LOGIN_FAILED) &&
      connection.account
    ) {
      return (
        <StyledIcon
          icon={ErrorNotif}
          size={24}
          className="konnector-state-picto"
        />
      )
    }

    if (outdatedData && outdatedData > 0) {
      return (
        <StyledIcon
          icon={WarningNotif}
          size={24}
          className="konnector-state-picto"
        />
      )
    }
  }

  const displayKonnectorIcon = useCallback(() => {
    return (
      <div className="konnector-icon">
        <Icon
          icon={currentFluidStatus.connection.account ? iconType : OfflinePicto}
          size={49}
        />
        {getIconForStatus(
          currentFluidStatus.status,
          currentFluidStatus.maintenance,
          currentFluidStatus.connection,
          isOutdatedData
        )}
      </div>
    )
  }, [currentFluidStatus, iconType, isOutdatedData])

  const displayKonnectorHeader = useCallback(() => {
    if (currentFluidStatus.maintenance) {
      return (
        <span className="text-16-bold">
          {t(`konnector_options.partner_issue`)}
        </span>
      )
    } else if (isOutdatedData && currentFluidStatus.connection.account) {
      return (
        <span className="text-16-bold outdated">
          {t('konnector_options.outdated', {
            isOutdatedData: isOutdatedData,
          })}
        </span>
      )
    } else if (currentFluidStatus.connection.account && !isOutdatedData) {
      return t(`FLUID.${FluidType[currentFluidStatus.fluidType]}.LABEL`)
    } else {
      return t(
        `konnector_options.label_offline_${FluidType[
          currentFluidStatus.fluidType
        ].toLowerCase()}`
      )
    }
  }, [
    currentFluidStatus.connection.account,
    currentFluidStatus.fluidType,
    currentFluidStatus.maintenance,
    isOutdatedData,
    t,
  ])

  // If user has selected accept button on Expired consent modal, then delete its account
  useEffect(() => {
    async function deleteAccountForConsentRefresh() {
      if (shouldRefreshConsent && account) {
        const accountService = new AccountService(client)
        await accountService.deleteAccount(account)
        await handleAccountDeletion()
      }
    }
    deleteAccountForConsentRefresh()
  }, [account, client, handleAccountDeletion, shouldRefreshConsent])

  useEffect(() => {
    let subscribed = true
    async function getData() {
      if (
        currentFluidStatus.connection.shouldLaunchKonnector &&
        !isKonnectorRunning(trigger)
      ) {
        if (subscribed) {
          if (currentFluidStatus.connection.isUpdating) setIsUpdating(true)
          setOpenModal(true)
          const updatedConnection: FluidConnection = {
            ...currentFluidStatus.connection,
            shouldLaunchKonnector: false,
          }
          dispatch(
            updateFluidConnection({
              fluidType: currentFluidStatus.fluidType,
              fluidConnection: updatedConnection,
            })
          )
        }
        const connectionFlow = new ConnectionFlow(client, trigger, konnector)
        await connectionFlow.launch()
        connectionFlow.jobWatcher.on(ERROR_EVENT, () => {
          sendUsageEventError(
            fluidSlug,
            currentFluidStatus.lastDataDate === null
          )
          setKonnectorErrorDescription(connectionFlow.jobWatcher.on()._error)
          callbackResponse(ERROR_EVENT)
        })
        connectionFlow.jobWatcher.on(LOGIN_SUCCESS_EVENT, () => {
          setIsLogging(false)
          sendUsageEventSuccess(
            fluidSlug,
            currentFluidStatus.lastDataDate === null
          )
        })
        connectionFlow.jobWatcher.on(SUCCESS_EVENT, () => {
          sendUsageEventSuccess(
            fluidSlug,
            currentFluidStatus.lastDataDate === null
          )
          callbackResponse(SUCCESS_EVENT)
        })
      }
    }
    !shouldRefreshConsent && getData()
    const dateChartService = new DateChartService()
    setIsOutdatedData(
      dateChartService.isDataOutdated(
        currentFluidStatus.lastDataDate,
        fluidType
      )
    )
    return () => {
      subscribed = false
    }
  }, [
    callbackResponse,
    client,
    currentFluidStatus.connection,
    currentFluidStatus.fluidType,
    currentFluidStatus.lastDataDate,
    dispatch,
    fluidSlug,
    fluidType,
    konnector,
    sendUsageEventError,
    sendUsageEventSuccess,
    shouldRefreshConsent,
    trigger,
  ])

  return (
    <>
      {!isDisconnected ? (
        <Accordion
          expanded={active}
          onChange={toggleAccordion}
          classes={{
            root: `expansion-panel-root ${
              !currentFluidStatus.maintenance &&
              (currentFluidStatus.status === FluidState.ERROR ||
                currentFluidStatus.status === FluidState.ERROR_LOGIN_FAILED)
                ? 'red-border'
                : ''
            }`,
          }}
        >
          <AccordionSummary
            aria-label={t(
              `konnector_options.accessibility.button_toggle_detail_${FluidType[
                currentFluidStatus.fluidType
              ].toLowerCase()}`
            )}
            expandIcon={
              <Icon icon={chevronDown} size={16} className="accordion-icon" />
            }
            classes={{
              root: 'expansion-panel-summary',
              content: 'expansion-panel-content',
            }}
          >
            {displayKonnectorIcon()}
            <div
              className={classNames('text-16-bold konnector-title', {
                [`${FluidType[
                  currentFluidStatus.fluidType
                ].toLowerCase()}-connected`]:
                  currentFluidStatus.status !== FluidState.NOT_CONNECTED &&
                  !currentFluidStatus.maintenance,
              })}
            >
              {displayKonnectorHeader()}
            </div>
          </AccordionSummary>
          <AccordionDetails
            classes={{
              root: 'expansion-panel-details',
            }}
          >
            {getConnectionCard()}
          </AccordionDetails>
        </Accordion>
      ) : (
        <>
          <AccordionDetails>{getConnectionCard()}</AccordionDetails>
        </>
      )}
      <KonnectorModal
        open={openModal}
        isUpdating={isUpdating}
        isLogging={isLogging}
        state={konnectorState}
        error={konnectorErrorDescription}
        fluidType={currentFluidStatus.fluidType}
        handleCloseClick={handleConnectionEnd}
        handleAccountDeletion={handleAccountDeletion}
        account={account}
      />
    </>
  )
}
export default KonnectorViewerCard
