/* eslint-disable camelcase */
import { QueryResult } from 'cozy-client'
import * as harvestLibAccounts from 'cozy-harvest-lib/dist/connections/accounts'
import { Account, AccountAuthData } from 'models'
import { accountsData } from 'tests/__mocks__/accountsData.mock'
import mockClient from 'tests/__mocks__/client.mock'
import { konnectorsData } from 'tests/__mocks__/konnectorsData.mock'
import { getError } from 'tests/__mocks__/testUtils'
import { triggersEnedisData } from 'tests/__mocks__/triggersData.mock'
import AccountService from './account.service'

jest.mock('cozy-harvest-lib/dist/connections/accounts')
const mockHavestLibAccounts = harvestLibAccounts as jest.Mocked<
  typeof harvestLibAccounts
>

const mockGetTriggerForAccount = jest.fn()
jest.mock('./triggers.service', () => {
  return jest.fn(() => ({
    getTriggerForAccount: mockGetTriggerForAccount,
  }))
})

describe('Account service', () => {
  const accountService = new AccountService(mockClient)

  describe('createAccount method', () => {
    it('should return created account for login type', async () => {
      mockHavestLibAccounts.createAccount.mockResolvedValueOnce(accountsData[2])
      const mockAuthData: AccountAuthData = {
        login: 'login',
        password: 'password', // NOSONAR
      }
      const result = await accountService.createAccount(
        konnectorsData[2],
        mockAuthData
      )
      expect(result).toEqual(accountsData[2])
    })
  })

  describe('getAccount method', () => {
    it('should return account', async () => {
      mockHavestLibAccounts.fetchAccount.mockResolvedValueOnce(accountsData[2])
      const mockId = 'io.cozy.konnectors/eglgrandlyon'
      const result = await accountService.getAccount(mockId)
      expect(result).toBe(accountsData[2])
    })

    it('should throw error when fetchAccount unsuccessfully', async () => {
      mockHavestLibAccounts.fetchAccount.mockRejectedValueOnce(new Error())
      const mockId = 'io.cozy.konnectors/eglgrandlyon'
      const error = await getError(async () =>
        accountService.getAccount(mockId)
      )
      expect(error).toEqual(new Error('Get account failed'))
    })
  })

  describe('getAccountByType method', () => {
    it('should return account with account_type equals to eglgrandlyon', async () => {
      const mockQueryResult: QueryResult<Account[]> = {
        data: [accountsData[2]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const mockType = 'eglgrandlyon'
      const result = await accountService.getAccountByType(mockType)
      expect(result).toEqual(accountsData[2])
    })

    it('should return null when no account find', async () => {
      const mockQueryResult: QueryResult<Account[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const mockType = 'eglgrandlyon'
      const result = await accountService.getAccountByType(mockType)
      expect(result).toBe(null)
    })
    it('should return the account linked to oldest trigger when several account', async () => {
      const mockType = 'enedissgegrandlyon'
      const mockAccounts = accountsData
      mockAccounts[1].account_type = mockType
      mockAccounts[2].account_type = mockType
      const mockQueryResult: QueryResult<Account[]> = {
        data: [...mockAccounts],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      mockGetTriggerForAccount.mockResolvedValueOnce(triggersEnedisData[0])
      mockGetTriggerForAccount.mockResolvedValueOnce(triggersEnedisData[1])
      mockGetTriggerForAccount.mockResolvedValueOnce(triggersEnedisData[2])
      const result = await accountService.getAccountByType(mockType)
      expect(result).toBe(mockAccounts[0])
    })
  })

  describe('getAccountsByType method', () => {
    it('should return all accounts for a type when several account', async () => {
      const mockType = 'enedissgegrandlyon'
      const mockAccounts = accountsData
      mockAccounts[1].account_type = mockType
      mockAccounts[2].account_type = mockType
      const mockQueryResult: QueryResult<Account[]> = {
        data: [...mockAccounts],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await accountService.getAccountsByType(mockType)
      expect(result).toEqual(mockAccounts)
    })
  })

  describe('updateAccount method', () => {
    it('should return updated account when updateAccount successfully', async () => {
      mockHavestLibAccounts.updateAccount.mockResolvedValueOnce(accountsData[2])
      const result = await accountService.updateAccount(accountsData[2])
      expect(result).toEqual(accountsData[2])
    })

    it('should throw error when updateAccount unsuccessfully', async () => {
      mockHavestLibAccounts.updateAccount.mockRejectedValueOnce(new Error())
      const error = await getError(async () =>
        accountService.updateAccount(accountsData[2])
      )
      expect(error).toEqual(new Error('Update account failed'))
    })
  })

  describe('deleteAccount method', () => {
    it('should return true when destroy successfully', async () => {
      mockHavestLibAccounts.deleteAccount.mockResolvedValueOnce(accountsData[2])
      const result = await accountService.deleteAccount(accountsData[2])
      expect(result).toBe(true)
    })

    it('should throw error when destroy unsuccessfully', async () => {
      mockHavestLibAccounts.deleteAccount.mockRejectedValueOnce(new Error())
      const error = await getError(async () =>
        accountService.deleteAccount(accountsData[2])
      )
      expect(error).toEqual(new Error('Delete account failed'))
    })
  })

  describe('createIndexAccount method', () => {
    it('should return empty array', async () => {
      const mockQueryResult: QueryResult<Account[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await accountService.createIndexAccount()
      expect(result).toEqual([])
    })
  })
})
