import * as Sentry from '@sentry/react'
import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import {
  Account,
  AccountAuthData,
  AccountSgeData,
  Konnector,
  SgeStore,
  Trigger,
} from 'models'
import AccountService from 'services/account.service'
import KonnectorService from 'services/konnector.service'
import TriggerService from 'services/triggers.service'
import logApp from 'utils/logger'

const logStack = logger.namespace('connectionService')

export default class ConnectionService {
  private _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  public async connectNewUser(
    konnectorId: string,
    eglAuthData?: AccountAuthData,
    sgeAuthData?: SgeStore
  ): Promise<{ account: Account; trigger: Trigger }> {
    // Retrieve konnector
    const konnectorService = new KonnectorService(this._client)
    const konnector: Konnector | null = await konnectorService.getKonnector(
      konnectorId
    )
    if (!konnector?.slug) {
      const errorMessage = `Could not find konnector for ${konnectorId}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw new Error(errorMessage)
    }
    // Creation of the account linked to the konnector retrieved
    let accountAuthData: AccountAuthData | AccountSgeData
    if (eglAuthData) {
      accountAuthData = {
        login: eglAuthData.login,
        password: eglAuthData.password,
      } as AccountAuthData
    } else {
      // No login and password for Sge account
      accountAuthData = {
        pointId: sgeAuthData?.pdl ? sgeAuthData.pdl.toString() : '',
        firstname: sgeAuthData?.firstName,
        lastname: sgeAuthData?.lastName,
        address: sgeAuthData?.address,
        postalCode: sgeAuthData?.zipCode ? sgeAuthData.zipCode.toString() : '',
        city: sgeAuthData?.city,
      } as AccountSgeData
    }
    const accountService = new AccountService(this._client)
    const account: Account = await accountService.createAccount(
      konnector,
      accountAuthData
    )
    if (!account?._id) {
      const errorMessage = `Error during account creation`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw new Error(errorMessage)
    }
    // creation of the trigger for the konnector retrieve and the created account
    const triggersServices = new TriggerService(this._client)
    const trigger: Trigger = await triggersServices.createTrigger(
      account,
      konnector
    )
    if (!trigger) {
      const errorMessage = `Error during trigger creation`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw new Error(errorMessage)
    }
    return {
      account: account,
      trigger: trigger,
    }
  }
}
